package com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory

import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.Atributo
import com.example.ellegadodepintia.exploradoresDePintia.model.Opcion
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoEscalar
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoExplorar
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoFuerzaBruta
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoPico

class TrampaFactory : EventoFactory {
    private val trampasMap = mapOf(
        "Un estruendoso derrumbe de piedras bloquea el camino, dejando un montón de rocas caídas que impiden tu avance." to Pair(
            R.drawable.asi_exploradores_trampa_piedras, listOf(
                Opcion(
                    descripcion = "Fuerza Bruta",
                    atributoRequerida = Atributo.Vida,
                    EfectoFuerzaBruta()
                ), Opcion(
                    descripcion = "Picar",
                    atributoRequerida = Atributo.Nula,
                    EfectoPico()
                ), Opcion(
                    descripcion = "Explorar",
                    atributoRequerida = Atributo.Investigacion,
                    EfectoExplorar()
                ), Opcion(
                    descripcion = "Escalar",
                    atributoRequerida = Atributo.Energia,
                    EfectoEscalar()
                )
            )
        ),

        )

    override fun generarDescripcion(): String {
        return trampasMap.keys.random()
    }

    override fun generarOpciones(descripcion: String): MutableList<Opcion> {
        return trampasMap[descripcion]?.second?.shuffled()?.take(4)?.toMutableList()
            ?: mutableListOf()
    }

    override fun generarImagen(descripcion: String): Int {
        return trampasMap[descripcion]?.first!!
    }

    override fun generarEvento(): EventoTrampa {
        val descripcion = generarDescripcion()
        return EventoTrampa(
            descripcion = descripcion,
            dificultad = 0,
            opciones = generarOpciones(descripcion),
            imagen = generarImagen(descripcion),
            resistenciaMax = (2..8).random(),
            finalizado = false
        )
    }
}