package com.example.ellegadodepintia.minijuego1

import android.animation.Animator
import android.animation.AnimatorListenerAdapter
import android.animation.ObjectAnimator
import android.content.Intent
import android.graphics.drawable.AnimationDrawable
import android.os.Bundle
import android.os.CountDownTimer
import android.os.Handler
import android.os.Looper
import android.view.animation.AccelerateDecelerateInterpolator
import android.view.animation.AlphaAnimation
import android.widget.Button
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.soundManager.SoundManager
import kotlin.random.Random

class Minijuego1 : AppCompatActivity() {

    // Variables para la interfaz de usuario y lógica del juego
    private lateinit var frameLayout: FrameLayout
    private lateinit var puntajeTextView: TextView
    private lateinit var tiempoTextView: TextView
    private lateinit var soundManager: SoundManager // Instancia de SoundManager
    private var puntaje = 0
    private var tiempoRestante = 30
    private var fuera = false
    private val handler = Handler(Looper.getMainLooper())

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_minijuego1)

        // Inicialización de vistas
        frameLayout = findViewById(R.id.frameLayout)
        puntajeTextView = findViewById(R.id.puntajeTextView)
        tiempoTextView = findViewById(R.id.tiempoTextView)
        soundManager = SoundManager(this)
        iniciarJuego()
        iniciarContador()
    }

    // Función principal para iniciar el juego
    private fun iniciarJuego() {
        soundManager.playSoundLoop(R.raw.sound_cueva)
        handler.postDelayed(object : Runnable {
            override fun run() {
                agregarBoton()
                handler.postDelayed(this, Random.nextLong(10, tiempoRestante.toLong() * 50))
            }
        }, 1000)
    }

    // Función para agregar un nuevo botón al juego
    private fun agregarBoton() {
        val boton = Button(this)
        val esHueso = Random.nextFloat() > 0.3

        // Configuración del botón
        if (esHueso) {
            boton.setBackgroundResource(R.drawable.asi_minijuego_1_hueso)
        } else {
            boton.setBackgroundResource(R.drawable.asi_minijuego_1_bomba)
        }
        boton.rotation = Random.nextInt(0, 360).toFloat()
        configurarPosicionBoton(boton)

        // Animación de aparición
        val fadeIn = AlphaAnimation(0f, 1f).apply {
            duration = 500
            fillAfter = true
        }
        boton.startAnimation(fadeIn)

        // Configurar acción al pulsar
        boton.setOnClickListener {
            actualizarPuntaje(esHueso)
            frameLayout.removeView(boton)
            mostrarParticulas(boton.x + boton.width / 2, boton.y + boton.height / 2, esHueso)
        }

        frameLayout.addView(boton)
        eliminarBotonConAnimacion(boton)
    }

    // Configuración de la posición aleatoria del botón
    private fun configurarPosicionBoton(boton: Button) {
        val huesoAleatorio = Random.nextInt(200, 400)
        val params = FrameLayout.LayoutParams(huesoAleatorio, huesoAleatorio)
        params.leftMargin = Random.nextInt(0, frameLayout.width - 300)
        params.topMargin = Random.nextInt(0, frameLayout.height - 300)
        boton.layoutParams = params
    }

    // Eliminar el botón después de un tiempo con una animación
    private fun eliminarBotonConAnimacion(boton: Button) {
        handler.postDelayed({
            val fadeOut = AlphaAnimation(1f, 0f).apply {
                duration = 500
                fillAfter = true
            }
            boton.startAnimation(fadeOut)
            handler.postDelayed({ frameLayout.removeView(boton) }, 500)
        }, Random.nextLong(800, 1000))
    }

    // Contador de tiempo
    private fun iniciarContador() {
        object : CountDownTimer(30 * 1000L, 1000) {
            override fun onTick(millisUntilFinished: Long) {
                tiempoRestante = (millisUntilFinished / 1000).toInt()
                tiempoTextView.text = "Tiempo: $tiempoRestante"
                if (tiempoRestante == 1) {
                    handler.removeCallbacksAndMessages(null)
                }
            }

            override fun onFinish() {
                val intent = Intent(this@Minijuego1, FinMinijuego1::class.java)
                intent.putExtra("puntuacion", puntaje)
                if (!fuera) {
                    startActivity(intent)
                }
                finish()
            }
        }.start()
    }

    // Actualizar el puntaje
    private fun actualizarPuntaje(esHueso: Boolean) {
        if (puntaje == 0 && !esHueso) {
            puntaje = 0
            puntajeTextView.text = "Puntaje: 0"
        } else if (esHueso) {
            puntaje++
        } else {
            puntaje--
        }
        puntajeTextView.text = "Puntaje: $puntaje"
    }

    override fun onPause() {
        super.onPause()
        fuera = true
        handler.removeCallbacksAndMessages(null)
        soundManager.stopSound()
    }

    override fun onDestroy() {
        super.onDestroy()
        handler.removeCallbacksAndMessages(null)
        soundManager.stopSound()
    }

    // Mostrar partículas o animación en la posición del botón tocado
    private fun mostrarParticulas(x: Float, y: Float, esHueso: Boolean) {
        if (esHueso) {
            soundManager.playSound(R.raw.sound_huesito) // Usando el SoundManager
            // Mostrar partículas si es un hueso
            val cantidadParticulas = Random.nextInt(5, 11)
            for (i in 0 until cantidadParticulas) {
                val particula = ImageView(this)
                particula.setImageResource(R.drawable.asi_minijuego_1_hueso)
                particula.layoutParams = FrameLayout.LayoutParams(25, 25)
                frameLayout.addView(particula)
                particula.x = x
                particula.y = y

                val animX = ObjectAnimator.ofFloat(particula, "translationX", x + Random.nextInt(-200, 200))
                val animY = ObjectAnimator.ofFloat(particula, "translationY", y + Random.nextInt(-200, 200))
                val alphaAnim = ObjectAnimator.ofFloat(particula, "alpha", 1f, 0f)

                animX.duration = Random.nextLong(300, 800)
                animY.duration = animX.duration
                alphaAnim.duration = animX.duration

                val interpolator = AccelerateDecelerateInterpolator()
                animX.interpolator = interpolator
                animY.interpolator = interpolator
                alphaAnim.interpolator = interpolator

                animX.start()
                animY.start()
                alphaAnim.start()

                alphaAnim.addListener(object : AnimatorListenerAdapter() {
                    override fun onAnimationEnd(animation: Animator) {
                        frameLayout.removeView(particula)
                    }
                })
            }
        } else {
            val explosionView = ImageView(this)
            soundManager.playSound(R.raw.sound_bomba) // Usando el SoundManager
            explosionView.setBackgroundResource(R.drawable.effect_reparar_animation)
            val params = FrameLayout.LayoutParams(300, 300)
            params.leftMargin = x.toInt() - 150
            params.topMargin = y.toInt() - 150
            explosionView.layoutParams = params
            frameLayout.addView(explosionView)

            // Iniciar la animación
            handler.postDelayed({
                (explosionView.background as? AnimationDrawable)?.start()
                explosionView.animate()
                    .alpha(0f)
                    .setDuration(1000L)
                    .withEndAction {
                        frameLayout.removeView(explosionView)
                    }
                    .start()
            }, 0)
        }
    }
}
