package com.example.ellegadodepintia.minijuego3

import android.animation.Animator
import android.animation.ObjectAnimator
import android.content.Intent
import android.os.Bundle
import android.os.CountDownTimer
import android.os.Handler
import android.os.Looper
import android.widget.Button
import android.widget.GridLayout
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import android.animation.AnimatorListenerAdapter
import android.animation.ValueAnimator
import android.graphics.ColorMatrix
import android.graphics.ColorMatrixColorFilter
import com.example.ellegadodepintia.MapActivity
import com.example.ellegadodepintia.R

class Minijuego3 : AppCompatActivity() {
    private lateinit var portadaLayout: LinearLayout
    private lateinit var botonIniciar: ImageView
    private lateinit var gridLayout: GridLayout
    private lateinit var cronometroTextView: TextView
    private lateinit var botonVolver: ImageView;

    private var botonesCarta = mutableListOf<Button>()
    private var cartas = mutableListOf<Carta>()
    private var primieraCarta: Carta? = null
    private var segundaCarta: Carta? = null
    private var procesandose = false
    private var cuentraAtras: CountDownTimer? = null
    private var tiempoRestante: Long = 45000 // 1 minuto en milisegundos
    private var tiempoCorriendo = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_minijuego3)
        inizializarVistas()
        setupBotonStart()
    }

    private fun inizializarVistas() {
        portadaLayout = findViewById(R.id.portadaLayout)
        botonIniciar = findViewById(R.id.btn_jugar)
        gridLayout = findViewById(R.id.gridLayout)
        cronometroTextView = findViewById(R.id.cronometro)
        botonVolver = findViewById(R.id.btn_volver)
    }

    private fun setupBotonStart() {
        botonIniciar.setOnClickListener {
            esconderPortada()
            iniciarJuego()
        }
    }

    private fun esconderPortada() {
        botonVolver.visibility = ImageView.GONE
        portadaLayout.visibility = LinearLayout.GONE
        gridLayout.visibility = GridLayout.VISIBLE
        cronometroTextView.visibility = TextView.VISIBLE
    }

    private fun iniciarJuego() {
        inizcializarJuego()
        iniciarCronometro()
        voltearCartas()
    }

    private fun inizcializarJuego() {
        val images = listOf(
            R.drawable.asi_minijuego_3_vasito, R.drawable.asi_minijuego_3_copa_negra_brunida, R.drawable.asi_minijuego_3_jarra_de_pico, R.drawable.asi_minijuego_3_copa_tumba_75,
            R.drawable.asi_minijuego_3_sonajero_redondo, R.drawable.asi_minijuego_3_tres_en_raya, R.drawable.asi_minijuego_3_tapadera_zoomorfo, R.drawable.asi_minijuego_3_tintinnabulum
        )
        val imagenesCartas = (images + images).shuffled()
        val tamannoCarta = 200
        for (i in 0 until 16) {
            val carta = Carta(id = i, imageResId = imagenesCartas[i])
            cartas.add(carta)
            val button = Button(this).apply {
                layoutParams = GridLayout.LayoutParams().apply {
                    width = tamannoCarta
                    height = tamannoCarta + 50
                    setMargins(20, 20, 20, 20)
                }
                setBackgroundResource(R.drawable.asi_minijuego_3_trasero)
                setOnClickListener { cartaClickada(i) }
            }
            botonesCarta.add(button)
            gridLayout.addView(button)
        }
    }

    private fun iniciarCronometro() {
        cuentraAtras?.cancel()
        cuentraAtras = object : CountDownTimer(tiempoRestante, 1000) {
            override fun onTick(millisUntilFinished: Long) {
                tiempoRestante = millisUntilFinished
                actualizarContador()
            }
            override fun onFinish() {
                mostrarGameOver("¡Has perdido!")
            }
        }.start()
        tiempoCorriendo = true
    }

    private fun actualizarContador() {
        val segundos = (tiempoRestante / 1000).toInt()
        val minutos = segundos / 60
        val segundosRestantes = segundos % 60
        cronometroTextView.text = String.format("%02d:%02d", minutos, segundosRestantes)
    }

    private fun cartaClickada(position: Int) {
        if (procesandose) return
        val carta = cartas[position]
        val button = botonesCarta[position]
        if (carta.isFaceUp || carta.isMatched) return
        carta.isFaceUp = true
        if (primieraCarta == null) {
            primieraCarta = carta
            girarCarta(button, carta.imageResId)
        } else if (segundaCarta == null) {
            segundaCarta = carta
            girarCarta(button, carta.imageResId)
            procesandose = true
            comprobarMatch()
        }
    }

    private fun comprobarMatch() {
        Handler(Looper.getMainLooper()).postDelayed({
            if (primieraCarta?.imageResId == segundaCarta?.imageResId) {
                primieraCarta?.isMatched = true
                segundaCarta?.isMatched = true
                sumarTiempo()
                primieraCarta?.let { card ->
                    val button = botonesCarta[cartas.indexOf(card)]
                    iluminarCarta(button)
                }
                segundaCarta?.let { card ->
                    val button = botonesCarta[cartas.indexOf(card)]
                    iluminarCarta(button)}
            } else {
                primieraCarta?.let { card ->
                    val button = botonesCarta[cartas.indexOf(card)]
                    card.isFaceUp = false
                    girarCarta(button, R.drawable.asi_minijuego_3_trasero, isReversing = true)
                }
                segundaCarta?.let { card ->
                    val button = botonesCarta[cartas.indexOf(card)]
                    card.isFaceUp = false
                    girarCarta(button, R.drawable.asi_minijuego_3_trasero, isReversing = true)
                }
            }
            primieraCarta = null
            segundaCarta = null
            procesandose = false
            comprobarFinJuego()
        }, 1000)
    }


    private fun comprobarFinJuego() {
        if (cartas.all { it.isMatched }) {
            tiempoCorriendo = false
            cuentraAtras?.cancel()
            mostrarGameOver("¡Has ganado!")
        }
    }

    private fun sumarTiempo() {
        tiempoRestante += 5000
        actualizarContador()
        iniciarCronometro()
    }

    private fun mostrarGameOver(mensaje: String) {
        Toast.makeText(this, mensaje, Toast.LENGTH_SHORT).show()
        botonVolver.visibility = ImageView.VISIBLE
        botonVolver.setOnClickListener{
            val intent = Intent(this@Minijuego3, MapActivity::class.java)
            startActivity(intent)
            finish()
        }
        gridLayout.removeAllViews()
    }

    override fun onBackPressed() {
        cuentraAtras?.cancel()
        tiempoCorriendo = false
        super.onBackPressed()
    }

    private fun girarCarta(button: Button, imageResId: Int, isReversing: Boolean = false) {
        val scale = applicationContext.resources.displayMetrics.density
        button.cameraDistance = 8000 * scale
        val flipOut = ObjectAnimator.ofFloat(button, "rotationY", 0f, 90f).apply {
            duration = 300
        }
        val flipIn = ObjectAnimator.ofFloat(button, "rotationY", 90f, 0f).apply {
            duration = 300
        }
        flipOut.addListener(object : AnimatorListenerAdapter() {
            override fun onAnimationEnd(animation: Animator) {
                button.setBackgroundResource(if (isReversing) R.drawable.asi_minijuego_3_trasero else imageResId)
                flipIn.start()
            }
        })
        flipOut.start()
    }

    private fun iluminarCarta(button: Button) {
        val matrix = ColorMatrix()
        matrix.setScale(1.5f, 1.5f, 1.5f, 1f)
        val colorFilter = ColorMatrixColorFilter(matrix)
        button.background.colorFilter = colorFilter
        val animator = ValueAnimator.ofFloat(1.5f, 1f)
        animator.duration = 1500
        animator.addUpdateListener { animation ->
            val value = animation.animatedValue as Float
            val scaleMatrix = ColorMatrix()
            scaleMatrix.setScale(value, value, value, 1f)
            button.background.colorFilter = ColorMatrixColorFilter(scaleMatrix)
        }
        animator.start()
    }

    private fun voltearCartas() {
        botonesCarta.forEachIndexed { index, button ->
            val card = cartas[index]
            card.isFaceUp = true
            girarCarta(button, card.imageResId)
        }
        Handler(Looper.getMainLooper()).postDelayed({
            botonesCarta.forEachIndexed { index, button ->
                val card = cartas[index]
                card.isFaceUp = false
                girarCarta(button, R.drawable.asi_minijuego_3_trasero, isReversing = true)
            }
        }, 2000)
    }
}
